/*!@file
** @brief   The entry point of the program.
** @remarks This file is the single compilation unit of the program.
** @author  UETA, Shin-ichi <ueta@usdesign.jp>
**
** @copyright (C) 2014-2018 Ueta System Design Studio\n
** This file is licensed under the terms of the MIT License.\n
** See <https://opensource.org/licenses/MIT>.
*/
#include "stdafx.h"
#include "resource.h"
#include "dialog.h"

/*!
** @brief   The global instance of CAppModule.
** @remarks -
*/
CAppModule _Module;

/*!
** @brief   Implements the entry point for the application.
** @param   [in] hInstance a handle to the current instance of the application.
** @param   [in] hPrevInstance a legacy from 16-bit Windows, now always zero.
** @param   [in] lpCmdLine the command-line for the application.
** @param   [in] nCmdShow specifies how the window is to be shown.
** @returns A standard HRESULT value as the exit value.
** @remarks -
*/
extern "C" int WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(nCmdShow);

	_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);

	HRESULT hResult = S_OK;

	hResult = ::CoInitialize(NULL);
	ATLASSERT(SUCCEEDED(hResult));
	::DefWindowProc(NULL, WM_NULL, 0, 0L);
	AtlInitCommonControls(ICC_WIN95_CLASSES | ICC_DATE_CLASSES);
	ATLASSERT(!CAtlBaseModule::m_bInitFailed);
	hResult = _Module.Init(NULL, hInstance);
	ATLASSERT(SUCCEEDED(hResult));

	CPath dir;
	::GetCurrentDirectory(MAX_PATH, dir.m_strPath.GetBuffer(MAX_PATH));
	dir.m_strPath.ReleaseBuffer();
	ATLASSERT(!dir.IsRelative());

	CPath path;

	bool bRun = true;

	if (__argc < 2) // no command-line arguments
	{
		path = dir;
	}
	else if (lstrcmpi(lpCmdLine, TEXT("/RegServer")) == 0)
	{
		hResult = _Module.UpdateRegistryFromResource(IDR_APP, TRUE);
		ATLASSERT(SUCCEEDED(hResult));
		bRun = false;
	}
	else if (lstrcmpi(lpCmdLine, TEXT("/UnregServer")) == 0)
	{
		hResult = _Module.UpdateRegistryFromResource(IDR_APP, FALSE);
		ATLASSERT(SUCCEEDED(hResult));
		bRun = false;
	}
	else
	{
		ATLASSERT(__targv);
		ATLASSERT(__targv[1]);
		path.Combine(dir, __targv[1]);
		ATLASSERT(!path.IsRelative());
		path.RemoveBackslash();
	}
	if (bRun)
	{
		CMessageLoop theMessageLoop;

		if (_Module.AddMessageLoop(&theMessageLoop))
		{
			CDialog theDialog(path);

			if (theDialog.Create(HWND_DESKTOP))
			{
				ATLASSERT(theDialog.IsWindowVisible());

				hResult = HRESULT(theMessageLoop.Run());
			}
			else
			{
				hResult = AtlHresultFromLastError();
			}
			ATLVERIFY(_Module.RemoveMessageLoop());
		}
		else
		{
			hResult = AtlHresultFromLastError();
		}
	}
	if (FAILED(hResult))
	{
		CString message = AtlGetErrorDescription(hResult);

		AtlMessageBox(NULL, LPCTSTR(message), IDR_APP, MB_ICONERROR);
	}
	_Module.Term();
	::CoUninitialize();
	return int(hResult);
}
